#include "handmade.h"

#define PI32 3.141592653589f

internal void renderSmallRect(GameOffscreenBuffer *buffer, int playerX, int playerY) {
    uint8 *endOfBuffer = (uint8 *)buffer->memory + buffer->pitch*buffer->height;
    uint32 color = 0xFFFFFFFF;
    int top = playerY;
    int bottom = playerY + 10;
    for (int x = playerX; x < playerX + 10; x++) {
        uint8 *pixel = ((uint8 *)buffer->memory + x * buffer->bytesPerPixel + top*buffer->pitch);
        for (int y = top; y < bottom; y++) {
            if ((pixel >= buffer->memory) && (pixel + 4 < endOfBuffer)) {
                *(uint32 *)pixel = color;
            }
            pixel += buffer->pitch;
        }
    }

}

internal void outputSineSound(GameSoundOutputBuffer *soundBuffer, GameState *state) {
    int16 toneVolume = 3000;
    int wavePeriod = soundBuffer->samplesPerSecond/state->toneHz;

#if 0
    int16 *sampleOut = soundBuffer->samples;
    for (int sampleIndex = 0; sampleIndex < soundBuffer->sampleCount; sampleIndex++) {
        state->tSine += 2.0f * PI32 / (real32)wavePeriod;
        int16 sampleValue = (int16)(sin(state->tSine) * (real32)toneVolume);
        *sampleOut++ = sampleValue;
        *sampleOut++ = sampleValue;
    }
#endif
}

internal void renderWeirdGradient(GameOffscreenBuffer *buffer, int xOffset, int yOffset) {
    uint8 *row = (uint8 *)buffer->memory;
    for (int y = 0; y < buffer->height; y++) {
        uint32 *pixel = (uint32*)row;
        for (int x = 0; x < buffer->width; x++) {
            uint8 blue = (uint8)(x + xOffset);
            uint8 green = (uint8)(y + yOffset);
            *pixel++ = (green << 8) | blue;
        }
        row += buffer->pitch;
    }
}

extern "C" GAME_UPDATE_AND_RENDER(gameUpdateAndRender) {
    Assert(sizeof(GameState) <= memory->permanentStorageSize);

    GameState *state = (GameState*)memory->permanentStorage;

    if (!memory->isInitialised) {
        DebugReadFileResult bmpMem = memory->debugReadEntireFile(ctx, __FILE__);
        if (bmpMem.contents) {
            if (false) {
                memory->debugWriteEntireFile(ctx, "c:/source/repos/handmade/src/test.cpp", bmpMem.contentsSize, bmpMem.contents);
            }
            memory->debugFreeFileMemory(ctx, bmpMem.contents);
        }

        state->toneHz = 440;
        state->tSine = 0;
        state->playerY = 100;
        state->playerX = 100;
        state->blueOffset = 0;
        state->greenOffset = 0;
        memory->isInitialised = true;
    }

    for (int controllerIndex = 0; controllerIndex < ArrayCount(input->controllers); controllerIndex++) {
        GameControllerInput *controllerInput = &input->controllers[controllerIndex];

        if (controllerInput->btnDown.endedDown) {
            state->playerY -= 30;
        }

        if (controllerInput->stickRight.endedDown) {
            state->playerX += 10;
        }
        if (controllerInput->stickLeft.endedDown) {
            state->playerX -= 10;
        }
        if (controllerInput->stickDown.endedDown) {
            state->playerY += 10;
        }
        if (controllerInput->stickUp.endedDown) {
            state->playerY -= 10;
        }
    }

    renderWeirdGradient(videoBuf, state->greenOffset, state->blueOffset);
    renderSmallRect(videoBuf, state->playerX, state->playerY);
    renderSmallRect(videoBuf, input->mouseX, input->mouseY);
    for (int i = 0; i < ArrayCount(input->mouseButtons); i++) {
        if (input->mouseButtons[i].endedDown) {
            renderSmallRect(videoBuf, 10 + i * 20, 10);
        }
    }
}

extern "C" GAME_GET_SOUND_SAMPLES(gameGetSoundSamples) {
    outputSineSound(soundBuf, (GameState*)memory->permanentStorage);
}
